use dotenv::dotenv;
use poise::{serenity_prelude as serenity, ApplicationContext};

mod errors;
mod death;
mod claude;
mod fal;

struct Data {
    claude: claude::Claude,
    fal: fal::Fal,
}
type Error = Box<dyn std::error::Error + Send + Sync>;
type Context<'a> = ApplicationContext<'a, Data, Error>;

/// Ask Huaxu anything!
#[poise::command(slash_command)]
pub(crate) async fn ask_huaxu(
    ctx: Context<'_>,
    #[description = "Your question"]
    question: String,
) -> Result<(), Error> {
    ctx.defer().await?;

    let msg = ctx.data.claude.ask_huaxu(&question, None).await?;
    ctx.reply(msg).await?;

    Ok(())
}

/// Ask Huaxu anything!
#[poise::command(slash_command)]
pub(crate) async fn ask_huaxu_genz(
    ctx: Context<'_>,
    #[description = "Your question"]
    question: String,
) -> Result<(), Error> {
    ctx.defer().await?;

    let msg = ctx.data.claude.ask_huaxu(&question, Some("Gen-Z")).await?;
    ctx.reply(msg).await?;

    Ok(())
}

/// Let huaxu dream up something
#[poise::command(slash_command)]
pub(crate) async fn genimage(
    ctx: Context<'_>,
    #[description = "Your prompt"]
    prompt: String,
) -> Result<(), Error> {
    ctx.defer().await?;

    let msg = ctx.data.fal.request("flux/dev", &prompt).await?;
    if msg.images.is_empty() {
        return Err("No images found".into());
    }

    let image = &msg.images[0];
    ctx.reply(&image.url).await?;

    Ok(())
}

#[tokio::main]
async fn main() {
    dotenv().ok();
    tracing_subscriber::fmt::init();

    let anthropic_key = std::env::var("ANTHROPIC_API_KEY").expect("missing ANTHROPIC_API_KEY");
    let token = std::env::var("DISCORD_TOKEN").expect("missing DISCORD_TOKEN");
    let fal_key = std::env::var("FAL_KEY").expect("missing FAL_KEY");
    let intents = serenity::GatewayIntents::non_privileged();

    let framework = poise::Framework::builder()
        .options(poise::FrameworkOptions {
            commands: vec![death::death_by_huaxu(), ask_huaxu(), ask_huaxu_genz(), genimage()],
            ..Default::default()
        })
        .setup(|ctx, _ready, framework| {
            Box::pin(async move {
                poise::builtins::register_globally(ctx, &framework.options().commands).await?;
                Ok(Data {
                    claude: claude::Claude::new(anthropic_key),
                    fal: fal::Fal::new(fal_key),
                })
            })
        })
        .build();

    let client = serenity::ClientBuilder::new(token, intents)
        .framework(framework)
        .await;
    client.unwrap().start().await.unwrap();
}
